----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity watchdog is
  generic(nbit_instr       :     integer := 16;
          nbit_signal      :     integer := 24;
          nbit_enc_piporom :     integer := 6
          );
  port ( clk               : in  std_logic;
         enable            : in  std_logic;
         instr_in          : in  std_logic_vector (nbit_instr-1 downto 0);
         rstn              : in  std_logic;
         signal_in         : in  std_logic_vector (nbit_signal-1 downto 0);
         ok                : out std_logic);
end watchdog;

architecture struct1 of watchdog is

  component AND2
    port ( I0 : in  std_logic;
           I1 : in  std_logic;
           O  : out std_logic);
  end component;

  component encoder_16_6
    generic (
      nbitin    :     integer := 16;
      nbitout   :     integer := 16);
    port ( in0  : in  std_logic_vector (nbitin-1 downto 0);
           out0 : out std_logic_vector (nbitout-1 downto 0));
  end component;

  component ff_d
    port ( D    : in  std_logic;
    		 EN   : in std_logic;
           CLK  : in  std_logic;
           RSTn : in  std_logic;
           Q    : out std_logic);
  end component;

  component mask
    generic (
      nbit      :     integer := 16);
    port ( in0  : in  std_logic_vector (nbit-1 downto 0);
           in1  : in  std_logic_vector (nbit-1 downto 0);
           out0 : out std_logic_vector (nbit-1 downto 0));
  end component;

  component pipo
    generic (
      nbit        :     integer := 16);
    port ( enable : in  std_logic;
           clk    : in  std_logic;
           rstn   : in  std_logic;
           in0    : in  std_logic_vector (nbit-1 downto 0);
           out0   : out std_logic_vector (nbit-1 downto 0));
  end component;

  component rom_mf
    generic (
      nbitin    :     integer := 6;
      nbitout   :     integer := 8);
    port ( in0  : in  std_logic_vector (nbitin-1 downto 0);
           out0 : out std_logic_vector (nbitout-1 downto 0));
  end component;

  component rom_se
    generic (
      nbitin    :     integer := 6;
      nbitout   :     integer := 8);
    port ( in0  : in  std_logic_vector (nbitin-1 downto 0);
           out0 : out std_logic_vector (nbitout-1 downto 0));
  end component;

  component rom_me
    generic (
      nbitin    :     integer := 6;
      nbitout   :     integer := 8);
    port ( in0  : in  std_logic_vector (nbitin-1 downto 0);
           out0 : out std_logic_vector (nbitout-1 downto 0));
  end component;

  component rom_sf
    generic (
      nbitin    :     integer := 6;
      nbitout   :     integer := 8);
    port ( in0  : in  std_logic_vector (nbitin-1 downto 0);
           out0 : out std_logic_vector (nbitout-1 downto 0));
  end component;

  component comparator
    generic (
      nbit      :     integer := 16);
    port ( inA  : in  std_logic_vector (nbit-1 downto 0);
           inB  : in  std_logic_vector (nbit-1 downto 0);
           out0 : out std_logic);
  end component;

  signal comp1_and   : std_logic;
  signal comp2_ff1   : std_logic;
  signal enc_piporom : std_logic_vector (nbit_enc_piporom-1 downto 0);
  signal ff1_and     : std_logic;
  signal mask1_comp1 : std_logic_vector (nbit_signal-1 downto 0);
  signal mask2_comp2 : std_logic_vector (nbit_signal-1 downto 0);
  signal pipo2_enc   : std_logic_vector (nbit_instr-1 downto 0);
  signal pipo3_rom   : std_logic_vector (nbit_enc_piporom-1 downto 0);
  signal rom1_comp1  : std_logic_vector (nbit_signal-1 downto 0);
  signal rom2_mask1  : std_logic_vector (nbit_signal-1 downto 0);
  signal rom3_com2   : std_logic_vector (nbit_signal-1 downto 0);
  signal rom4_mask2  : std_logic_vector (nbit_signal-1 downto 0);
  signal and_ff2     : std_logic;
  signal pipo4_mask  : std_logic_vector (nbit_signal-1 downto 0);
  signal pipo1_pipo2 : std_logic_vector (nbit_instr-1 downto 0);
  signal enable_delayed : std_logic;

begin
  AND1 : AND2
    port map (I0 => ff1_and,
              I1 => comp1_and,
              O  => and_ff2);

  ENC1 : encoder_16_6
    generic map (
      nbitin       => nbit_instr,
      nbitout      => nbit_enc_piporom)
    port map (in0  => pipo2_enc,
              out0 => enc_piporom);

  FFD1 : ff_d
    port map (CLK  => clk,
    		    EN   => enable_delayed,
              D    => comp2_ff1,
              RSTn => rstn,
              Q    => ff1_and);

  MASK1 : mask
    generic map (
      nbit         => nbit_signal)
    port map (in0  => rom2_mask1,
              in1  => pipo4_mask,
              out0 => mask1_comp1);

  MASK2 : mask
    generic map (
      nbit         => nbit_signal)
    port map (in0  => rom4_mask2,
              in1  => pipo4_mask,
              out0 => mask2_comp2);

  PIPO1 : pipo
    generic map (
      nbit           => nbit_instr)
    port map (clk    => clk,
              enable => enable,
              in0    => instr_in,
              rstn   => rstn,
              out0   => pipo1_pipo2);

  PIPO2 : pipo
    generic map (
      nbit           => nbit_instr)
    port map (clk    => clk,
              enable => enable,
              in0    => pipo1_pipo2,
              rstn   => rstn,
              out0   => pipo2_enc);

  PIPO3 : pipo
    generic map (
      nbit           => nbit_enc_piporom)
    port map (clk    => clk,
              enable => enable,
              in0    => enc_piporom,
              rstn   => rstn,
              out0   => pipo3_rom);
 
  FFD3 : ff_d
    port map (CLK  => clk,
    		    EN   => '1',
              D    => enable,
              RSTn => rstn,
              Q    => enable_delayed);

  PIPO4 : pipo
    generic map (
      nbit           => nbit_signal)
    port map (clk    => clk,
              enable => enable_delayed,
              in0    => signal_in,
              rstn   => rstn,
              out0   => pipo4_mask);

  ROM1 : rom_se
    generic map (
      nbitin       => nbit_enc_piporom,
      nbitout      => nbit_signal)
    port map (in0  => pipo3_rom,
              out0 => rom1_comp1);

  ROM2 : rom_me
    generic map (
      nbitin       => nbit_enc_piporom,
      nbitout      => nbit_signal)
    port map (in0  => pipo3_rom,
              out0 => rom2_mask1);

  ROM3 : rom_sf
    generic map (
      nbitin       => nbit_enc_piporom,
      nbitout      => nbit_signal)
    port map (in0  => enc_piporom,
              out0 => rom3_com2);

  ROM4 : rom_mf
    generic map (
      nbitin       => nbit_enc_piporom,
      nbitout      => nbit_signal)
    port map (in0  => enc_piporom,
              out0 => rom4_mask2);

  COMP1 : comparator
    generic map (
      nbit         => nbit_signal)
    port map (inA  => rom1_comp1,
              inB  => mask1_comp1,
              out0 => comp1_and);

  COMP2 : comparator
    generic map (
      nbit         => nbit_signal)
    port map (inA  => rom3_com2,
              inB  => mask2_comp2,
              out0 => comp2_ff1);

  FFD2 : ff_d
    port map (CLK  => clk,
    		    EN   => enable_delayed,
              D    => and_ff2,
              RSTn => rstn,
              Q    => ok);

end struct1;


