----------------------------------------------------------------------------
----                                                                    ----
----  This file is part of the WATCHDOG project                         ----
----                                                                    ----
----  Description:                                                      ----
----  Implementation of a WATCHDOG for RISCMCU                          ----
----                                                                    ----
----                                                                    ----
----------------------------------------------------------------------------
----                                                                    ----
----  Copyright (C) 2006 P. Sarro, C. Sasso, G. Scappatura              ----
----                                                                    ----
----  This source file may be used and distributed without              ----
----  restriction provided that this copyright statement is not         ----
----  removed from the file and that any derivative work contains       ----
----  the original copyright notice and the associated disclaimer.      ----
----                                                                    ----
----  This source file is free software; you can redistribute it        ----
----  and/or modify it under the terms of the GNU Lesser General        ----
----  Public License as published by the Free Software Foundation;      ----
----  either version 2.1 of the License, or (at your option) any        ----
----  later version.                                                    ----
----                                                                    ----
----  This source is distributed in the hope that it will be            ----
----  useful, but WITHOUT ANY WARRANTY; without even the implied        ----
----  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           ----
----  PURPOSE. See the GNU Lesser General Public License for more       ----
----  details.                                                          ----
----                                                                    ----
----  A copy of the license can be found at                             ----
----  http://www.fsf.org/licensing/licenses/lgpl.html                   ----
----                                                                    ----
----------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-------------------------------------------------------------------------------

entity watchdog_tb is

end watchdog_tb;

-------------------------------------------------------------------------------

architecture test of watchdog_tb is

  component watchdog
    generic (
      nbit_instr       :     integer;
      nbit_signal      :     integer;
      nbit_enc_piporom :     integer);
    port (
      clk              : in  std_logic;
      enable           : in  std_logic;
      instr_in         : in  std_logic_vector (nbit_instr-1 downto 0);
      rstn             : in  std_logic;
      signal_in        : in  std_logic_vector (nbit_signal-1 downto 0);
      ok               : out std_logic);
  end component;

  constant nbit_instr       : integer := 16;
  constant nbit_signal      : integer := 24;
  constant nbit_enc_piporom : integer := 6;

  signal clk       : std_logic:='0';
  signal enable    : std_logic:='0';
  signal instr_in  : std_logic_vector (nbit_instr-1 downto 0);
  signal rstn      : std_logic:='1';
  signal signal_in : std_logic_vector (nbit_signal-1 downto 0);
  signal ok        : std_logic;

  constant clk_period : time := 84 ns;
  constant delay      : time := clk_period-clk_period/4;
  
begin  -- test

  DUT : watchdog
    generic map (
      nbit_instr       => nbit_instr,
      nbit_signal      => nbit_signal,
      nbit_enc_piporom => nbit_enc_piporom)
    port map (
      clk              => clk,
      enable           => enable,
      instr_in         => instr_in,
      rstn             => rstn,
      signal_in        => signal_in,
      ok               => ok);

  rstn <= '0' after clk_period, '1' after 2*clk_period;
  clk <= not(clk) after clk_period/2;

  enable   <= '0',
              '1' after  2*clk_period, -- rjmp  (3 clk)
              '0' after  3*clk_period,
              '1' after  5*clk_period, -- ret   (3 clk)
              '0' after  6*clk_period,
              '1' after  8*clk_period, -- ret   (3 clk)
              '0' after  9*clk_period,
              '1' after 11*clk_period, -- clr   (1 clk)
              '1' after 12*clk_period, -- com   (1 clk)
              '1' after 13*clk_period, -- out   (1 clk)
              '1' after 14*clk_period, -- out   (1 clk)
              '1' after 15*clk_period, -- out   (1 clk)
              '1' after 16*clk_period, -- ldi   (1 clk)
              '1' after 17*clk_period, -- out   (1 clk)
              '1' after 18*clk_period, -- dec   (1 clk)
              '1' after 19*clk_period, -- cpi   (1 clk)
              '1' after 20*clk_period, -- brne  (3 clk)
              '0' after 21*clk_period,
              '1' after 23*clk_period, -- clr   (1 clk)
              '1' after 24*clk_period, -- inc   (1 clk)
              '1' after 25*clk_period, -- inc   (1 clk)
              '1' after 26*clk_period, -- inc   (1 clk)
              '1' after 27*clk_period, -- out   (1 clk)
              '1' after 28*clk_period, -- subi  (1 clk)
              '1' after 29*clk_period, -- cpse  (2 clk)
              '0' after 30*clk_period,
              '1' after 31*clk_period, -- rjmp  (3 clk)
              '0' after 32*clk_period,
              '1' after 34*clk_period, -- clr   (1 clk)
              '1' after 35*clk_period, -- subi  (1 clk)
              '1' after 36*clk_period, -- out   (1 clk)
              '1' after 37*clk_period, -- dec   (1 clk)
              '1' after 38*clk_period, -- in    (1 clk)
              '1' after 39*clk_period, -- sbrs  (2 clk)
              '0' after 40*clk_period,
              '1' after 41*clk_period, -- rjmp  (3 clk)
              '0' after 42*clk_period,
              '1' after 44*clk_period, -- rjmp  (3 clk)
              '0' after 45*clk_period,
              '1' after 47*clk_period, -- sei   (1 clk)
              '1' after 48*clk_period; -- set   (1 clk)
                                     
  instr_in <= X"C002" after  2*clk_period, -- rjmp  (3 clk)
              X"9508" after  5*clk_period, -- ret   (3 clk)
              X"9508" after  8*clk_period, -- ret   (3 clk)
              X"2733" after 11*clk_period, -- clr   (1 clk)
              X"9530" after 12*clk_period, -- com   (1 clk)
              X"BB37" after 13*clk_period, -- out   (1 clk)
              X"BB34" after 14*clk_period, -- out   (1 clk)
              X"BB31" after 15*clk_period, -- out   (1 clk)
              X"E003" after 16*clk_period, -- ldi   (1 clk)
              X"BB08" after 17*clk_period, -- out   (1 clk)
              X"950A" after 18*clk_period, -- dec   (1 clk)
              X"3000" after 19*clk_period, -- cpi   (1 clk)
              X"F7E1" after 20*clk_period, -- brne  (3 clk)
              X"2733" after 23*clk_period, -- clr   (1 clk)
              X"9513" after 24*clk_period, -- inc   (1 clk)
              X"9513" after 25*clk_period, -- inc   (1 clk)
              X"9513" after 26*clk_period, -- inc   (1 clk)
              X"BB15" after 27*clk_period, -- out   (1 clk)
              X"5011" after 28*clk_period, -- subi  (1 clk)
              X"1313" after 29*clk_period, -- cpse  (2 clk)
              X"CFFC" after 31*clk_period, -- rjmp  (3 clk)
              X"2722" after 34*clk_period, -- clr   (1 clk)
              X"5F2D" after 35*clk_period, -- subi  (1 clk)
              X"BB22" after 36*clk_period, -- out   (1 clk)
              X"952A" after 37*clk_period, -- dec   (1 clk)
              X"B73F" after 38*clk_period, -- in    (1 clk)
              X"FF31" after 39*clk_period, -- sbrs  (2 clk)
              X"CFFB" after 41*clk_period, -- rjmp  (3 clk)
              X"CFFF" after 44*clk_period, -- rjmp  (3 clk)
              X"9478" after 47*clk_period, -- sei   (1 clk)
              X"9468" after 48*clk_period; -- set   (1 clk)
  
  --tutte corrette 
--  signal_in <= "------0-----------------" after  2*clk_period+delay, -- rjmp  (3 clk)
--               "1-----------1---------0-" after  5*clk_period+delay, -- ret   (3 clk)
--               "111-------1-1-1-1-01----" after  8*clk_period+delay, -- ret   (3 clk)
--               "111-------1-1-1-1-01----" after 11*clk_period+delay, -- clr   (1 clk)
--               "---------------1-101---0" after 12*clk_period+delay, -- com   (1 clk)
--               "-1-0--0------------1--0-" after 13*clk_period+delay, -- out   (1 clk)
--               "---0--0-0---------------" after 14*clk_period+delay, -- out   (1 clk)
--               "---0--0-0---------------" after 15*clk_period+delay, -- out   (1 clk)
--               "--------0------11----0--" after 16*clk_period+delay, -- ldi   (1 clk)
--               "--10--0-----------------" after 17*clk_period+delay, -- out   (1 clk)
--               "--------01----1---------" after 18*clk_period+delay, -- dec   (1 clk)
--               "1-1------------1--------" after 19*clk_period+delay, -- cpi   (1 clk)
--               "1-------------1---------" after 20*clk_period+delay, -- brne  (3 clk)
--               "11----------------------" after 23*clk_period+delay, -- clr   (1 clk)
--               "--------0---1--1-10-----" after 24*clk_period+delay, -- inc   (1 clk)
--               "-1------0---1-----------" after 25*clk_period+delay, -- inc   (1 clk)
--               "-1------0---1-----------" after 26*clk_period+delay, -- inc   (1 clk)
--               "-1-0--0-----------------" after 27*clk_period+delay, -- out   (1 clk)
--               "--------0---1-----------" after 28*clk_period+delay, -- subi  (1 clk)
--               "----1-0-0----1---1--0---" after 29*clk_period+delay, -- cpse  (2 clk)
--               "---0--0----0------------" after 31*clk_period+delay, -- rjmp  (3 clk)
--               "1---------------------0-" after 34*clk_period+delay, -- clr   (1 clk)
--               "------------1--1-10-----" after 35*clk_period+delay, -- subi  (1 clk)
--               "---0--0-0----1---1--0---" after 36*clk_period+delay, -- out   (1 clk)
--               "--------01----1---------" after 37*clk_period+delay, -- dec   (1 clk)
--               "1-----0---------1---0---" after 38*clk_period+delay, -- in    (1 clk)
--               "-----1--0-----1----1-0--" after 39*clk_period+delay, -- sbrs  (2 clk)
--               "----1-0--1-----1---1----" after 41*clk_period+delay, -- rjmp  (3 clk)
--               "------0---------------0-" after 44*clk_period+delay, -- rjmp  (3 clk)
--               "------0---1-------0--00-" after 47*clk_period+delay, -- sei   (1 clk)
--               "1-1---0---1---11--0--000" after 48*clk_period+delay; -- set   (1 clk)
                
  --tutte sbagliate 
  signal_in <= "------1-----------------" after  2*clk_period+delay, -- rjmp  (3 clk)
               "0-----------0---------1-" after  5*clk_period+delay, -- ret   (3 clk)
               "000-------0-0-0-0-10----" after  8*clk_period+delay, -- ret   (3 clk)
               "000-------0-0-0-0-10----" after 11*clk_period+delay, -- clr   (1 clk)
               "---------------0-010---1" after 12*clk_period+delay, -- com   (1 clk)
               "-0-1--1------------0--1-" after 13*clk_period+delay, -- out   (1 clk)
               "---1--1-1---------------" after 14*clk_period+delay, -- out   (1 clk)
               "---1--1-1---------------" after 15*clk_period+delay, -- out   (1 clk)
               "--------1------00----1--" after 16*clk_period+delay, -- ldi   (1 clk)
               "--01--1-----------------" after 17*clk_period+delay, -- out   (1 clk)
               "--------10----0---------" after 18*clk_period+delay, -- dec   (1 clk)
               "0-0------------0--------" after 19*clk_period+delay, -- cpi   (1 clk)
               "0-------------0---------" after 20*clk_period+delay, -- brne  (3 clk)
               "00----------------------" after 23*clk_period+delay, -- clr   (1 clk)
               "--------1---0--0-01-----" after 24*clk_period+delay, -- inc   (1 clk)
               "-0------1---0-----------" after 25*clk_period+delay, -- inc   (1 clk)
               "-0------1---0-----------" after 26*clk_period+delay, -- inc   (1 clk)
               "-0-1--1-----------------" after 27*clk_period+delay, -- out   (1 clk)
               "--------1---0-----------" after 28*clk_period+delay, -- subi  (1 clk)
               "----0-1-1----0---0--1---" after 29*clk_period+delay, -- cpse  (2 clk)
               "---1--1----1------------" after 31*clk_period+delay, -- rjmp  (3 clk)
               "0---------------------1-" after 34*clk_period+delay, -- clr   (1 clk)
               "------------0--0-01-----" after 35*clk_period+delay, -- subi  (1 clk)
               "---1--1-1----0---0--1---" after 36*clk_period+delay, -- out   (1 clk)
               "--------10----0---------" after 37*clk_period+delay, -- dec   (1 clk)
               "0-----1---------0---1---" after 38*clk_period+delay, -- in    (1 clk)
               "-----0--1-----0----0-1--" after 39*clk_period+delay, -- sbrs  (2 clk)
               "----0-1--0-----0---0----" after 41*clk_period+delay, -- rjmp  (3 clk)
               "------1---------------1-" after 44*clk_period+delay, -- rjmp  (3 clk)
               "------1---0-------1--11-" after 47*clk_period+delay, -- sei   (1 clk)
               "0-0---1---0---00--1--111" after 48*clk_period+delay; -- set   (1 clk)
                
  --sbagliata una fetch 
--  signal_in <= "------0-----------------" after  2*clk_period+delay, -- rjmp  (3 clk)
--               "1-----------1---------0-" after  5*clk_period+delay, -- ret   (3 clk)
--               "111-------1-1-1-1-01----" after  8*clk_period+delay, -- ret   (3 clk)
--               "111-------1-1-1-1-01----" after 11*clk_period+delay, -- clr   (1 clk)
--               "---------------1-101---0" after 12*clk_period+delay, -- com   (1 clk)
--               "-1-0--0------------1--0-" after 13*clk_period+delay, -- out   (1 clk)
--               "---0--0-0---------------" after 14*clk_period+delay, -- out   (1 clk)
--               "---0--0-0---------------" after 15*clk_period+delay, -- out   (1 clk)
--               "--------0------11----0--" after 16*clk_period+delay, -- ldi   (1 clk)
--               "--10--0-----------------" after 17*clk_period+delay, -- out   (1 clk)
--               "--------01----1---------" after 18*clk_period+delay, -- dec   (1 clk)
--               "1-1------------1--------" after 19*clk_period+delay, -- cpi   (1 clk)
--               "1-------------1---------" after 20*clk_period+delay, -- brne  (3 clk)
--               "11----------------------" after 23*clk_period+delay, -- clr   (1 clk)
--               "--------0---1--1-10-----" after 24*clk_period+delay, -- inc   (1 clk)
--               "-1------0---1-----------" after 25*clk_period+delay, -- inc   (1 clk)
--               "-1------0---1-----------" after 26*clk_period+delay, -- inc   (1 clk)
--               "-1-0--0-----------------" after 27*clk_period+delay, -- out   (1 clk)
--               "--------0---1-----------" after 28*clk_period+delay, -- subi  (1 clk)
--               "----0-0-0----1---1--0---" after 29*clk_period+delay, -- cpse  (2 clk) (ERRORE FETCH CPSE)
--               "---0--0----0------------" after 31*clk_period+delay, -- rjmp  (3 clk)
--               "1---------------------0-" after 34*clk_period+delay, -- clr   (1 clk)
--               "------------1--1-10-----" after 35*clk_period+delay, -- subi  (1 clk)
--               "---0--0-0----1---1--0---" after 36*clk_period+delay, -- out   (1 clk)
--               "--------01----1---------" after 37*clk_period+delay, -- dec   (1 clk)
--               "1-----0---------1---0---" after 38*clk_period+delay, -- in    (1 clk)
--               "-----1--0-----1----1-0--" after 39*clk_period+delay, -- sbrs  (2 clk)
--               "----1-0--1-----1---1----" after 41*clk_period+delay, -- rjmp  (3 clk)
--               "------0---------------0-" after 44*clk_period+delay, -- rjmp  (3 clk)
--               "------0---1-------0--00-" after 47*clk_period+delay, -- sei   (1 clk)
--               "1-1---0---1---11--0--000" after 48*clk_period+delay; -- set   (1 clk)
                
  --sbagliata una execute 
--  signal_in <= "------0-----------------" after  2*clk_period+delay, -- rjmp  (3 clk)
--               "1-----------1---------0-" after  5*clk_period+delay, -- ret   (3 clk)
--               "111-------1-1-1-1-01----" after  8*clk_period+delay, -- ret   (3 clk)
--               "111-------1-1-1-1-01----" after 11*clk_period+delay, -- clr   (1 clk)
--               "---------------1-101---0" after 12*clk_period+delay, -- com   (1 clk)
--               "-1-0--0------------1--0-" after 13*clk_period+delay, -- out   (1 clk)
--               "---0--0-0---------------" after 14*clk_period+delay, -- out   (1 clk)
--               "---0--0-0---------------" after 15*clk_period+delay, -- out   (1 clk)
--               "--------0------11----0--" after 16*clk_period+delay, -- ldi   (1 clk)
--               "--10--0-----------------" after 17*clk_period+delay, -- out   (1 clk)
--               "--------01----1---------" after 18*clk_period+delay, -- dec   (1 clk)
--               "1-1------------1--------" after 19*clk_period+delay, -- cpi   (1 clk)
--               "1-------------1---------" after 20*clk_period+delay, -- brne  (3 clk)
--               "10----------------------" after 23*clk_period+delay, -- clr   (1 clk) (ERRORE EXECUTE BRNE)
--               "--------0---1--1-10-----" after 24*clk_period+delay, -- inc   (1 clk)
--               "-1------0---1-----------" after 25*clk_period+delay, -- inc   (1 clk)
--               "-1------0---1-----------" after 26*clk_period+delay, -- inc   (1 clk)
--               "-1-0--0-----------------" after 27*clk_period+delay, -- out   (1 clk)
--               "--------0---1-----------" after 28*clk_period+delay, -- subi  (1 clk)
--               "----1-0-0----1---1--0---" after 29*clk_period+delay, -- cpse  (2 clk)
--               "---0--0----0------------" after 31*clk_period+delay, -- rjmp  (3 clk)
--               "1---------------------0-" after 34*clk_period+delay, -- clr   (1 clk)
--               "------------1--1-10-----" after 35*clk_period+delay, -- subi  (1 clk)
--               "---0--0-0----1---1--0---" after 36*clk_period+delay, -- out   (1 clk)
--               "--------01----1---------" after 37*clk_period+delay, -- dec   (1 clk)
--               "1-----0---------1---0---" after 38*clk_period+delay, -- in    (1 clk)
--               "-----1--0-----1----1-0--" after 39*clk_period+delay, -- sbrs  (2 clk)
--               "----1-0--1-----1---1----" after 41*clk_period+delay, -- rjmp  (3 clk)
--               "------0---------------0-" after 44*clk_period+delay, -- rjmp  (3 clk)
--               "------0---1-------0--00-" after 47*clk_period+delay, -- sei   (1 clk)
--               "1-1---0---1---11--0--000" after 48*clk_period+delay; -- set   (1 clk)
                

end test;

-------------------------------------------------------------------------------
